@@ -1,13 +1,13 @@
 package Quantum::Superpositions::Lazy::Superposition;
 
-our $VERSION = '1.09';
+our $VERSION = '1.10';
 
 use v5.24;
 use warnings;
 use Moo;
 use Quantum::Superpositions::Lazy::State;
 use Quantum::Superpositions::Lazy::Computation;
-use Quantum::Superpositions::Lazy::Util qw(get_rand is_collapsible);
+use Quantum::Superpositions::Lazy::Util qw(get_rand);
 use Types::Standard qw(ArrayRef InstanceOf);
 use List::Util qw(sum0);
 
@@ -76,9 +76,14 @@ sub reset
 {
 	my ($self) = @_;
 
-	foreach my $state ($self->_states->@*) {
+	if (!$self->{_collapsible}) {
+		$self->{_collapsible} = [grep { $_->collapsible } $self->_states->@*];
+	}
+
+	foreach my $state ($self->{_collapsible}->@*) {
 		$state->reset;
 	}
+
 	$self->_reset;
 
 	return $self;
@@ -88,20 +93,62 @@ sub _observe
 {
 	my ($self) = @_;
 
-	my @positions = $self->_states->@*;
-	my $sum = $self->weight_sum;
-	my $prob = get_rand;
+	if (!$self->{_lookup}) {
+		my @positions = $self->_states->@*;
+		my $sum = $self->weight_sum;
+		my @weights;
+		my $current = 0;
 
-	foreach my $state (@positions) {
-		$prob -= $state->weight / $sum;
-		if ($prob < 0) {
-			return is_collapsible($state->value)
-				? $state->value->collapse
-				: $state->value;
+		for my $state (@positions) {
+			push @weights, $current;
+			$current += $state->weight / $sum;
 		}
+
+		$self->{_lookup} = \@weights;
 	}
 
-	return undef;
+	my $prob = get_rand;
+
+	my @cache = $self->{_lookup}->@*;
+	my $current = int(@cache / 2);
+	my $step = int(@cache / 4) || 1;
+
+	# warn "rand: $prob, all: [@cache], step: $step";
+	while ($step > 1) {
+		if ($cache[$current] < $prob) {
+			# warn "going up: $current + $step";
+			$current += $step;
+
+			$current = @cache - 1
+				if $current >= @cache;
+		}
+
+		else {
+			# warn "going down $current - $step";
+			$current -= $step;
+
+			$current = 0
+				if $current < 0;
+		}
+
+		$step = int($step / 2);
+	}
+
+	while ($current < $#cache && $cache[$current] < $prob) {
+		$current += 1
+	}
+
+	while ($current > 0 && $cache[$current] > $prob) {
+		$current -= 1
+	}
+
+	# warn "selected: $current ($cache[$current] < $prob)";
+
+	my $state = $self->_states->[$current];
+
+	return $state->collapsible
+		? $state->value->collapse
+		: $state->value;
 }
 
 sub _build_complete_states
@@ -114,7 +161,7 @@ sub _build_complete_states
 		my $coeff = 1;
 
 		my $value = $state->value;
-		if (is_collapsible $value) {
+		if ($state->collapsible) {
 
 			# all values from this state must have their weights multiplied by $coeff
 			# this way the weight sum will stay the same
@@ -317,3 +364,4 @@ L<Quantum::Superpositions::Lazy/FUNCTIONS>.
 
 	neg + - * ** << >> / % += -= *= **= <<= >>= /= %= . x .=
 	x= atan2 cos sin exp log sqrt int abs
+
